import tkinter as tk
from PIL import ImageTk, Image
import rules 
import game

class Chessboard:
    def __init__(self,root):
        self.root = root
        self.root.title("My First Board")
        self.root.geometry("800x800")
        self.COLOR_BLACK_SQUARE = "#c87b45"
        self.COLOR_WHITE_SQUARE = "#ffdfac"
        self.piece_selected = None
        self.piece_position = None
        self.player_turn = 0
        self.square_size = 100
        self.ratio_piece = 0.8
        self.white_king_position = (7,4)
        self.black_king_position = (0,4)
        self.color_number = {0:'white',1:'black'}
        self.possible_moves = []
        self.white_castle = False
        self.black_castle = False
        self.mat = False
        self.en_passant = [[0 for k in range(8)] for p in range(2)]
        self.can_en_passant = False
        self.white_check = False
        self.black_check = False
        self._check_for_mat()

        self.piece_images = {
            "black-rook": ImageTk.PhotoImage(Image.open("Pieces/black-rook.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "black-knight": ImageTk.PhotoImage(Image.open("Pieces/black-knight.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "black-bishop": ImageTk.PhotoImage(Image.open("Pieces/black-bishop.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "black-queen": ImageTk.PhotoImage(Image.open("Pieces/black-queen.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "black-king": ImageTk.PhotoImage(Image.open("Pieces/black-king.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "black-pawn": ImageTk.PhotoImage(Image.open("Pieces/black-pawn.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "white-rook": ImageTk.PhotoImage(Image.open("Pieces/white-rook.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "white-knight": ImageTk.PhotoImage(Image.open("Pieces/white-knight.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "white-bishop": ImageTk.PhotoImage(Image.open("Pieces/white-bishop.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "white-queen": ImageTk.PhotoImage(Image.open("Pieces/white-queen.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "white-king": ImageTk.PhotoImage(Image.open("Pieces/white-king.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
            "white-pawn": ImageTk.PhotoImage(Image.open("Pieces/white-pawn.png").resize((int(self.square_size * self.ratio_piece), int(self.square_size * self.ratio_piece)))),
                                             
        }

        self.board = [[None for k in range(8)] for k in range(8)] #Variable de stockage du contrôle du jeu
        self.initial_positions = {
            (0, 0): "black-rook", (0, 1): "black-knight", (0, 2): "black-bishop",
            (0, 3): "black-queen", (0, 4): "black-king", (0, 5): "black-bishop",
            (0, 6): "black-knight", (0, 7): "black-rook",
            (1, 0): "black-pawn", (1, 1): "black-pawn", (1, 2): "black-pawn",
            (1, 3): "black-pawn", (1, 4): "black-pawn", (1, 5): "black-pawn",
            (1, 6): "black-pawn", (1, 7): "black-pawn",
            (6, 0): "white-pawn", (6, 1): "white-pawn", (6, 2): "white-pawn",
            (6, 3): "white-pawn", (6, 4): "white-pawn",(6, 6): "white-pawn", (6, 5): "white-pawn",
             (6, 7): "white-pawn",
            (7, 0): "white-rook", (7, 1): "white-knight", (7, 6): "white-knight",(7, 2): "white-bishop",
            (7, 3): "white-queen", (7, 4): "white-king", (7, 5): "white-bishop",
             (7, 7): "white-rook",
        }
        
        
        self.promotion = {(0, 0): "black-rook", (0, 1): "black-knight", (1, 1): "black-bishop",
            (1, 0): "black-queen"}
        
        self.PIECE_MAP = {
            'rook': rules.Rook,    # White Pawn
            'bishop': rules.Bishop,    # Black Pawn
            'king': rules.King,  # White Knight
            'queen': rules.Queen,  # Black Knight
            'pawn': rules.Pawn,  # White Bishop
            'knight': rules.Knight,  # Black Bishop
        }

        #add pieces to board
        for position, piece in self.initial_positions.items():
            self.board[position[0]][position[1]] = piece


        """self.board = game.Normalize(self.board,'rnbqkbnr/pp2pp1p/2p3p1/3p4/4NB2/5N2/PP2PPPP/2RQKB1R').board_extracted
"""

        self.canva = tk.Canvas(self.root, width=8*self.square_size, height=8*self.square_size)
        self.canva.bind("<ButtonPress-1>",self._on_square_click)
        self.canva.bind("<ButtonPress-2>",self._show_moves_piece)
        self.canva.pack()
        
        self.square_name ={0:'h',1:'g',2:'f',3:'e',4:'d',5:'c',6:'b',7:'a'}
        self.visual_moves ={}
        #Create the visual board from the matrix
        
        for row in range(8):
            for col in range(8):
                self.canva.create_rectangle(row*self.square_size, col*self.square_size, (row+1)*self.square_size, (col+1)*self.square_size, fill=self.COLOR_WHITE_SQUARE if (row + col) % 2 == 0 else self.COLOR_BLACK_SQUARE)
                
        self.piece_canva_id = {}
        for row in range(8):
            for col in range(8):
                if self.board[row][col]:
                    canva_piece= self.canva.create_image(col*self.square_size + self.square_size//2, row*self.square_size + self.square_size//2,image = self.piece_images[self.board[row][col]])
                    self.piece_canva_id[(row,col)] = canva_piece


        for row in range(8):
            self.canva.create_text(10,(row+1)*self.square_size -85,  text=str(8-row) , fill="black", font=('Helvetica 15 bold',20))

        for col in range(8):
            self.canva.create_text((col+1)*self.square_size -15, 785 ,text=self.square_name[8-col-1], fill="black", font=('Helvetica 15 bold',20))

    def _on_square_click(self,event): 
        #On update la matrice du jeu
        for value in self.visual_moves:
            self.canva.delete(self.visual_moves[value])
        self.visual_moves = {}
        row,col = self._get_square_from_click(event)
        if self.piece_selected:
            if (row,col) in self.possible_moves:

                #capture
                if self.board[row][col]:
                    self._remove_piece_visual(row,col) 

                #en-passant
                if (row,col) in self._en_passant_moves():
                    print('En-Passant')
                    self._make_en_passant()

                if self.piece_selected[6::] == 'pawn' and abs(self.piece_position[0] - row) ==2:
                    self._update_en_passant(row,col)
                else:
                    self.can_en_passant = False
                    
                #promotion pion: 
                if self.piece_selected[6::] == 'pawn' and 7 *self.player_turn == row:
                        print('le pion doit promote !')
                        self._promotion_method(row,col)

                #rock
                if (row,col) in self._castles_moves():
                    print('vous avez rocké')
                    self._make_castle(row,col)

                
                self.board[self.piece_position[0]][self.piece_position[1]] = None
                self.board[row][col] = self.piece_selected

                #on update la position du roi si nécessaire
                if self.piece_selected[6::]=='king':
                    if self.player_turn == 0: #on a joué un roi blanc
                        self.white_king_position = (row,col)
                        self.white_castle = True
                    else:
                        self.black_king_position = (row,col)
                        self.black_castle = True


                self._remove_piece_visual(self.piece_position[0],self.piece_position[1])
                self._draw_piece_visual(row,col,self.piece_selected)

                #afficher si ce coup met en échec l'adversaire
                if self._is_check(self.board,self.piece_selected,self.piece_position ,(row,col),1-self.player_turn):
                    print('Échec!')
                    if self.player_turn == 1:
                        self.white_check = True
                    elif self.player_turn ==0:
                        self.black_check = True
                self.piece_selected = None
                self.player_turn = 1- self.player_turn
                
                if self.player_turn ==0:
                    print('White to play')
                else:
                    print('Black to play')
                print(game.Normalize(self.board,'rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR').fen)
                print(game.Normalize(self.board,'rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR').board_extracted)
                
            else:
                self.piece_selected = None
                print('le coup est impossible, veuillez en choisir un autre')

        else:
            self._is_checkkmate()
            self._show_moves_piece(event)             

    def _promotion_method(self,row,col):
        self.promotion_choice = tk.Toplevel(self.root)
        self.promotion_choice.geometry("200x200")
        canva = tk.Canvas(self.promotion_choice, width=2*self.square_size, height=2*self.square_size)
        canva.pack()
        canva.bind("<ButtonPress-1>", lambda event, arg1=row, arg2=col: self._wanted_piece_promotion(event, arg1, arg2))
        for position, piece in self.promotion.items():
            canva.create_image(position[1]*self.square_size + self.square_size//2, position[0]*self.square_size + self.square_size//2, image=self.piece_images[f'{self.color_number[self.player_turn]}-{piece[6::]}'])
                        
    
    def _show_moves_piece(self,event): #montre visuellement les moves possibles et les stock pour le coup choisi
        row,col = self._get_square_from_click(event)
        accepted_moves = []
        if self.board[row][col]:
                if self.board[row][col][0:5]==self.color_number[self.player_turn]:
                    self.possible_moves = []
                    self.piece_selected = self.board[row][col]
                    self.piece_position = (row, col)
                    piece_class = self.PIECE_MAP[self.piece_selected[6::]]
                    piece = piece_class(self.piece_selected[0:5], self.piece_position,self.board)
                    self.possible_moves = piece.find_moves() #enlever les coups qui mettent le roi en échec 

                    for a in self.possible_moves:
                        if not self._is_check(self.board,self.piece_selected,self.piece_position,a,self.player_turn):
                            accepted_moves.append(a)

                    self.possible_moves = accepted_moves

                    #ajouter le rock:
                    if self.piece_selected[6::] == 'king':
                        moves = self._castles_moves()
                        for move in moves:
                            self.possible_moves.append(move)

                    #ajouter le en-passant
                    if self.piece_selected[6::] == 'pawn':
                        moves = self._en_passant_moves()
                        for move in moves:
                            self.possible_moves.append(move)

                    self._draw_possible_moves_visuals(self.possible_moves)
                else:
                    print('c est au tour de l autre joueur')

        else:
            print('rien dans cette case')
        return len(accepted_moves)

    
    def _castles_moves(self):
        move = []
        if self.player_turn == 0 and self.white_castle == False: #rock du joueur blanc
            #grand rock
            if self.board[7][2]== None and self.board[7][3]== None:
                if not self._is_check(self.board, self.piece_selected, self.piece_position, (7,2), self.player_turn) and not self._is_check(self.board, self.piece_selected, self.piece_position, (7,3), self.player_turn): 
                    move.append((7,2))
            #petit rock
            if self.board[7][5]== None and self.board[7][6]== None:
                if not self._is_check(self.board, self.piece_selected, self.piece_position, (7,5), self.player_turn) and not self._is_check(self.board, self.piece_selected, self.piece_position, (7,6), self.player_turn): 
                    move.append((7,6))

        elif self.player_turn == 1 and self.black_castle == False: #rock du joueur noir
            #grand rock
            if self.board[0][2]== None and self.board[0][3]== None:
                if not self._is_check(self.board, self.piece_selected, self.piece_position, (0,2), self.player_turn) and not self._is_check(self.board, self.piece_selected, self.piece_position, (0,3), self.player_turn): 
                    move.append((0,2))
            #petit rock
            if self.board[0][5]== None and self.board[0][6]== None:
                if not self._is_check(self.board, self.piece_selected, self.piece_position, (0,5), self.player_turn) and not self._is_check(self.board, self.piece_selected, self.piece_position, (0,6), self.player_turn): 
                    move.append((0,6))
                    
        return move
    
    def _en_passant_moves(self):#board du en passant potentiel
        move = []
        board_passant = [row[:] for row in self.board]
        col = None
        for k in range(len(self.en_passant[1-self.player_turn])):
            if self.en_passant[1-self.player_turn][k] ==1:
                col = k

        #Blancs        
        if self.player_turn ==0:
            if self.can_en_passant and self.piece_selected == 'white-pawn' and self.en_passant[self.player_turn][self.piece_position[1]]==1:
                board_passant[3][col]= None
                if not self._is_check(board_passant,'white-pawn',(self.piece_position[0],self.piece_position[1]),(2,col),self.player_turn):
                    move.append((2,col))
        #Noirs
        else:
            if self.can_en_passant and self.piece_selected == 'black-pawn' and self.en_passant[self.player_turn][self.piece_position[1]]==1:
                board_passant[3][col]= None
                if not self._is_check(board_passant,'black-pawn',(self.piece_position[0],self.piece_position[1]),(5,col),self.player_turn):
                    move.append((5,col))
        return move

    
    def _make_en_passant(self):
        col = None
        for k in range(len(self.en_passant[1-self.player_turn])):
            if self.en_passant[1-self.player_turn][k] ==1:
                col = k


        if self.player_turn ==0:
            self.board[3][col]= None
            self.board[self.piece_position[0]][self.piece_position[1]] = None
            self.board[2][col]= f'{self.color_number[self.player_turn]}-pawn'
            self._remove_piece_visual(3,col)

        else:
            self.board[4][col]= None
            self.board[self.piece_position[0]][self.piece_position[1]] = None
            self.board[5][col]= f'{self.color_number[self.player_turn]}-pawn'
            self._remove_piece_visual(4,col)

    def _make_castle(self,row,col):
        #grand rock
        if col == 2:
            self.board[7 - 7 *self.player_turn][0] = None
            self.board[7 - 7 *self.player_turn][3] = f'{self.color_number[self.player_turn]}-rook'
            self._draw_piece_visual(row,3,f'{self.color_number[self.player_turn]}-rook')
            self._remove_piece_visual(7 - 7 *self.player_turn,0)

        #petit rock
        if col == 6:
            self.board[7 - 7 *self.player_turn][7] = None
            self.board[7 - 7 *self.player_turn][5] = f'{self.color_number[self.player_turn]}-rook'
            self._draw_piece_visual(row,5,f'{self.color_number[self.player_turn]}-rook')
            self._remove_piece_visual(7 - 7 *self.player_turn,7)

        if self.player_turn ==0:
            self.white_castle = True
        if self.player_turn ==1:
            self.black_castle =True
        
    def _wanted_piece_promotion(self,event,row,col):
        row_temp,col_temp = self._get_square_from_click(event)
        self.board[row][col] = f'{self.color_number[1-self.player_turn]}-{self.promotion[(row_temp,col_temp)][6::]}'
        self._remove_piece_visual(row,col)
        self._draw_piece_visual(row,col,f'{self.color_number[1-self.player_turn]}-{self.promotion[(row_temp,col_temp)][6::]}')
        self.promotion_choice.destroy()
    
    def _update_en_passant(self,row,col):
        self.en_passant = [[0 for k in range(8)] for p in range(2)]
        if self.piece_position[1]>0 and self.piece_position[1]<7:

            if self.board[row][self.piece_position[1]+1] == f'{self.color_number[1-self.player_turn]}-pawn':
                self.en_passant[1-self.player_turn][self.piece_position[1]+1] = 1
                self.en_passant[self.player_turn][col]=1
                self.can_en_passant = True

            elif self.board[row][self.piece_position[1]-1] == f'{self.color_number[1-self.player_turn]}-pawn':
                self.en_passant[1-self.player_turn][self.piece_position[1]-1]= 1
                self.en_passant[self.player_turn][col]=1
                self.can_en_passant = True

            else:
                self.can_en_passant = False

        elif self.piece_position[1]==0:
            if self.board[row][self.piece_position[1]+1] == f'{self.color_number[1-self.player_turn]}-pawn':
                self.en_passant[1-self.player_turn][self.piece_position[1]+1] = 1
                self.en_passant[self.player_turn][col]=1
                self.can_en_passant = True

        elif self.piece_position[1]==7:
            if self.board[row][self.piece_position[1]-1] == f'{self.color_number[1-self.player_turn]}-pawn':
                self.en_passant[1-self.player_turn][self.piece_position[1]-1] = 1
                self.en_passant[self.player_turn][col]=1
                self.can_en_passant = True

    def _get_square_from_click(self,event):
        col, row = event.x//self.square_size, event.y//self.square_size
        return row,col
    
    def _draw_piece_visual(self,row,col,piece):
        canva_piece= self.canva.create_image(col*self.square_size + self.square_size//2, row*self.square_size + self.square_size//2,image = self.piece_images[piece])
        self.piece_canva_id[(row,col)] = canva_piece

    def _draw_possible_moves_visuals(self,list_moves):
        for moves in list_moves:
            row, col = moves[0], moves[1]
            canva_piece= self.canva.create_oval(col*self.square_size , row*self.square_size ,(col+1)*self.square_size , (row+1)*self.square_size )
            self.visual_moves[(row,col)] = canva_piece
   
    def _remove_piece_visual(self,row,col):
        self.canva.delete(self.piece_canva_id[(row,col)])
        self.piece_canva_id.pop((row,col))

    def _check_for_mat(self):
        if self.mat:
            print("Game over. Closing the window.")
            self.root.destroy()
        else:
            # Schedule the next check after 100 ms
            self.root.after(100, self._check_for_mat)

    def _is_check(self,board,piece,pos,move,player_turn): #vérifier si ce coup met un des rois en échec, retourne True si oui
        board_updated = [row[:] for row in board]  # Deep copy of the board
        board_updated[move[0]][move[1]] = piece
        board_updated[pos[0]][pos[1]] = None
        if piece[6::]!= 'king':
            for row in range(8):
                for col in range(8):
                    if board_updated[row][col] and board_updated[row][col][0:5] != self.color_number[player_turn]: #on regarde si il y a une pièce adverse et on check ses mouvemets possibles
                        possible_moves_adv = []
                        piece_adv = board_updated[row][col]
                        piece_position = (row, col)
                        piece_class = self.PIECE_MAP[piece_adv[6::]]
                        piece = piece_class(piece_adv[0:5], piece_position,board_updated)
                        possible_moves_adv = piece.find_moves()
                        #si ce sont les blancs qui ont joué: 
                        if player_turn == 0:
                            if self.white_king_position in possible_moves_adv:
                                return True
                        else:
                            if self.black_king_position in possible_moves_adv:
                                return True
                            
        else:
            for row in range(8):
                for col in range(8):
                    if board_updated[row][col] and board_updated[row][col][0:5] != self.color_number[player_turn]: #on regarde si il y a une pièce adverse et on check ses mouvemets possibles
                        possible_moves_adv = []
                        piece_adv = board_updated[row][col]
                        piece_position = (row, col)
                        piece_class = self.PIECE_MAP[piece_adv[6::]]
                        piece = piece_class(piece_adv[0:5], piece_position,board_updated)
                        possible_moves_adv = piece.find_moves()
                        
                        if move in possible_moves_adv:
                            return True
        return False

    def _is_checkkmate(self):
        if self.white_check: #on est en échec, on regarde si on est mat
                flag = False
                for row in range(8):
                    for col in range(8):
                        if self.board[row][col] and self.board[row][col][0:5] == self.color_number[self.player_turn]:
                            n = self._show_moves_piece_bis(row,col)
                            if n!=0:
                                flag = True
                if flag == False:
                    self.mat = True
                    print('mat les noirs gagnent')
                
                else:
                    self.white_check=False

        if self.black_check: #on est en échec, on regarde si on est mat
            flag = False
            for row in range(8):
                for col in range(8):
                    if self.board[row][col] and self.board[row][col][0:5] == self.color_number[self.player_turn]:
                        n = self._show_moves_piece_bis(row,col)
                        if n!=0:
                            flag = True
            if flag == False:
                self.mat = True
                print('mat les blancs gagnent')
            else:
                self.black_check=False

    def _show_moves_piece_bis(self,row,col): #montre visuellement les moves possibles et les stock pour le coup choisi
        accepted_moves = []
        if self.board[row][col]:
                if self.board[row][col][0:5]==self.color_number[self.player_turn]:
                    possible_moves = []
                    piece_selected = self.board[row][col]
                    piece_position = (row, col)                    
                    piece_class = self.PIECE_MAP[piece_selected[6::]]
                    piece = piece_class(piece_selected[0:5],piece_position,self.board)
                    possible_moves = piece.find_moves() #enlever les coups qui mettent le roi en échec 

                    for a in possible_moves:
                        if not self._is_check(self.board,piece_selected,self.piece_position ,a,self.player_turn):
                            accepted_moves.append(a)
                    possible_moves = accepted_moves
        return len(accepted_moves)
                
if __name__ == "__main__":
    root = tk.Tk()
    app = Chessboard(root)
    root.mainloop()
